MODULE ldfslp
   !!======================================================================
   !!                       ***  MODULE  ldfslp  ***
   !! Ocean physics: slopes of neutral surfaces
   !!======================================================================
   !! History :  OPA  ! 1994-12  (G. Madec, M. Imbard)  Original code
   !!            8.0  ! 1997-06  (G. Madec)  optimization, lbc
   !!            8.1  ! 1999-10  (A. Jouzeau)  NEW profile in the mixed layer
   !!   NEMO     0.5  ! 2002-10  (G. Madec)  Free form, F90
   !!            1.0  ! 2005-10  (A. Beckmann)  correction for s-coordinates
   !!----------------------------------------------------------------------
#if   defined key_ldfslp   ||   defined key_esopa
   !!----------------------------------------------------------------------
   !!   'key_ldfslp'                      Rotation of lateral mixing tensor
   !!----------------------------------------------------------------------
   !!   ldf_slp      : compute the slopes of neutral surface
   !!   ldf_slp_mxl  : compute the slopes of iso-neutral surface
   !!   ldf_slp_init : initialization of the slopes computation
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE ldftra_oce
   USE ldfdyn_oce
   USE phycst          ! physical constants
   USE zdfmxl          ! mixed layer depth
   USE eosbn2
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE in_out_manager  ! I/O manager
   USE prtctl          ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC   ldf_slp    ! routine called by step.F90
   PUBLIC ldf_slp_grif   !              "

   LOGICAL , PUBLIC, PARAMETER              ::   lk_ldfslp = .TRUE.   !: slopes flag
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   uslp, wslpi          !: i_slope at U- and W-points
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   vslp, wslpj          !: j-slope at V- and W-points
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   wslp2                !: wslp**2 from Griffies quarter cells
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   alpha, beta          !: alpha,beta at T points
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   tfw,sfw,ftu,fsu,ftv,fsv,ftud,fsud,ftvd,fsvd
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   psix_eiv
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   psiy_eiv
   
   REAL(wp), DIMENSION(jpi,jpj,jpk) ::   omlmask           ! mask of the surface mixed layer at T-pt
   REAL(wp), DIMENSION(jpi,jpj)     ::   uslpml, wslpiml   ! i_slope at U- and W-points just below the mixed layer
   REAL(wp), DIMENSION(jpi,jpj)     ::   vslpml, wslpjml   ! j_slope at V- and W-points just below the mixed layer

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "ldftra_substitute.h90"
#  include "ldfeiv_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.2 , LOCEAN-IPSL (2009) 
   !! $Id$
   !! Software governed by the CeCILL licence  (NEMOGCM/License_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE ldf_slp( kt, prd, pn2 )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE ldf_slp  ***
      !! 
      !! ** Purpose :   Compute the slopes of neutral surface (slope of isopycnal
      !!              surfaces referenced locally) ('key_traldfiso').
      !! 
      !! ** Method  :   The slope in the i-direction is computed at U- and 
      !!      W-points (uslp, wslpi) and the slope in the j-direction is 
      !!      computed at V- and W-points (vslp, wslpj).
      !!      They are bounded by 1/100 over the whole ocean, and within the
      !!      surface layer they are bounded by the distance to the surface
      !!      ( slope<= depth/l  where l is the length scale of horizontal
      !!      diffusion (here, aht=2000m2/s ==> l=20km with a typical velocity
      !!      of 10cm/s)
      !!        A horizontal shapiro filter is applied to the slopes
      !!        ln_sco=T, s-coordinate, add to the previously computed slopes
      !!      the slope of the model level surface.
      !!        macro-tasked on horizontal slab (jk-loop)  (2, jpk-1)
      !!      [slopes already set to zero at level 1, and to zero or the ocean
      !!      bottom slope (ln_sco=T) at level jpk in inildf]
      !!
      !! ** Action : - uslp, wslpi, and vslp, wslpj, the i- and  j-slopes 
      !!               of now neutral surfaces at u-, w- and v- w-points, resp.
      !!----------------------------------------------------------------------
      USE oce , zgru  => ua   ! use ua as workspace
      USE oce , zgrv  => va   ! use va as workspace
      USE oce , zwy   => ta   ! use ta as workspace
      USE oce , zwz   => sa   ! use sa as workspace
      !!
      INTEGER , INTENT(in)                         ::   kt    ! ocean time-step index
      REAL(wp), INTENT(in), DIMENSION(jpi,jpj,jpk) ::   prd   ! in situ density
      REAL(wp), INTENT(in), DIMENSION(jpi,jpj,jpk) ::   pn2   ! Brunt-Vaisala frequency (locally ref.)
      !!
      INTEGER  ::   ji , jj , jk    ! dummy loop indices
      INTEGER  ::   ii0, ii1, iku   ! temporary integer
      INTEGER  ::   ij0, ij1, ikv   ! temporary integer
      REAL(wp) ::   zeps, zmg, zm05g, zalpha        ! temporary scalars
      REAL(wp) ::   zcoef1, zcoef2, zcoef3          !    -         -
      REAL(wp) ::   zcofu , zcofv , zcofw           !    -         -
      REAL(wp) ::   zau, zbu, zai, zbi, z1u, z1wu   !    -         -
      REAL(wp) ::   zav, zbv, zaj, zbj, z1v, z1wv   !
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zww     ! 3D workspace
      !!----------------------------------------------------------------------
      
      IF( kt == nit000 )   CALL ldf_slp_init      ! initialization (first time-step only)

      zeps  =  1.e-20                             ! Local constant initialization
      zmg   = -1.0 / grav
      zm05g = -0.5 / grav
      !
      zww(:,:,:) = 0.e0
      zwz(:,:,:) = 0.e0
      !                                           ! horizontal density gradient computation
      DO jk = 1, jpk
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               zgru(ji,jj,jk) = umask(ji,jj,jk) * ( prd(ji+1,jj  ,jk) - prd(ji,jj,jk) ) 
               zgrv(ji,jj,jk) = vmask(ji,jj,jk) * ( prd(ji  ,jj+1,jk) - prd(ji,jj,jk) ) 
            END DO
         END DO
      END DO
      IF( ln_zps ) THEN                           ! partial steps correction at the bottom ocean level
# if defined key_vectopt_loop  
         DO jj = 1, 1
            DO ji = 1, jpij-jpi   ! vector opt. (forced unrolling)
# else
         DO jj = 1, jpjm1
            DO ji = 1, jpim1
# endif
               iku = MIN ( mbathy(ji,jj), mbathy(ji+1,jj) ) - 1               ! last ocean level
               ikv = MIN ( mbathy(ji,jj), mbathy(ji,jj+1) ) - 1
               zgru(ji,jj,iku) = gru(ji,jj) 
               zgrv(ji,jj,ikv) = grv(ji,jj)               
            END DO
         END DO
      ENDIF
      
      CALL ldf_slp_mxl( prd, pn2 )                ! Slopes of isopycnal surfaces just below the mixed layer

      
      ! I.  slopes at u and v point      | uslp = d/di( prd ) / d/dz( prd )
      ! ===========================      | vslp = d/dj( prd ) / d/dz( prd )
      !               
      !                                           !* Local vertical density gradient evaluated from N^2
      DO jk = 2, jpkm1                            !  zwy = d/dz(prd)= - ( prd ) / grav * mk(pn2) -- at t point
         DO jj = 1, jpj
            DO ji = 1, jpi
               zwy(ji,jj,jk) = zmg * ( prd(ji,jj,jk) + 1. ) * (    pn2  (ji,jj,jk) + pn2  (ji,jj,jk+1)     )   &
                  &                                         / MAX( tmask(ji,jj,jk) + tmask(ji,jj,jk+1), 1. )
            END DO
         END DO
      END DO
      DO jk = 2, jpkm1                            !* Slopes at u and v points
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               ! horizontal and vertical density gradient at u- and v-points
               zau = 1. / e1u(ji,jj) * zgru(ji,jj,jk)
               zav = 1. / e2v(ji,jj) * zgrv(ji,jj,jk)
               zbu = 0.5 * ( zwy(ji,jj,jk) + zwy(ji+1,jj  ,jk) )
               zbv = 0.5 * ( zwy(ji,jj,jk) + zwy(ji  ,jj+1,jk) )
               ! bound the slopes: abs(zw.)<= 1/100 and zb..<0
               !                   kxz max= ah slope max =< e1 e3 /(pi**2 2 dt)
               zbu = MIN( zbu, -100.*ABS( zau ), -7.e+3/fse3u(ji,jj,jk)*ABS( zau ) )
               zbv = MIN( zbv, -100.*ABS( zav ), -7.e+3/fse3v(ji,jj,jk)*ABS( zav ) )
               ! uslp and vslp output in zwz and zww, resp.
               zalpha = MAX( omlmask(ji,jj,jk), omlmask(ji+1,jj,jk) )
               zwz (ji,jj,jk) = ( ( 1. - zalpha) * zau / ( zbu - zeps )                                           &
                  &                    + zalpha  * uslpml(ji,jj)                                                  &
                  &                              * 0.5 * ( fsdept(ji+1,jj,jk)+fsdept(ji,jj,jk)-fse3u(ji,jj,1) )   &
                  &                              / MAX( hmlpt(ji,jj), hmlpt(ji+1,jj), 5. ) ) * umask(ji,jj,jk)
               zalpha = MAX( omlmask(ji,jj,jk), omlmask(ji,jj+1,jk) )
               zww (ji,jj,jk) = ( ( 1. - zalpha) * zav / ( zbv - zeps )                                           &
                  &                    + zalpha  * vslpml(ji,jj)                                                  &
                  &                              * 0.5 * ( fsdept(ji,jj+1,jk)+fsdept(ji,jj,jk)-fse3v(ji,jj,1) )   &
                  &                              / MAX( hmlpt(ji,jj), hmlpt(ji,jj+1), 5. ) ) * vmask(ji,jj,jk)
            END DO
         END DO
      END DO
      CALL lbc_lnk( zwz, 'U', -1. )   ;   CALL lbc_lnk( zww, 'V', -1. )      ! lateral boundary conditions
      !
      zcofu = 1. / 16.                            !* horizontal Shapiro filter
      zcofv = 1. / 16.
      DO jk = 2, jpkm1
         DO jj = 2, jpjm1, jpj-3                        ! rows jj=2 and =jpjm1 only
            DO ji = 2, jpim1  
               uslp(ji,jj,jk) = zcofu * (        zwz(ji-1,jj-1,jk) + zwz(ji+1,jj-1,jk)      &
                  &                       +      zwz(ji-1,jj+1,jk) + zwz(ji+1,jj+1,jk)      &
                  &                       + 2.*( zwz(ji  ,jj-1,jk) + zwz(ji-1,jj  ,jk)      &
                  &                       +      zwz(ji+1,jj  ,jk) + zwz(ji  ,jj+1,jk) )    &
                  &                       + 4.*  zwz(ji  ,jj  ,jk)                       )
               vslp(ji,jj,jk) = zcofv * (        zww(ji-1,jj-1,jk) + zww(ji+1,jj-1,jk)      &
                  &                       +      zww(ji-1,jj+1,jk) + zww(ji+1,jj+1,jk)      &
                  &                       + 2.*( zww(ji  ,jj-1,jk) + zww(ji-1,jj  ,jk)      &
                  &                       +      zww(ji+1,jj  ,jk) + zww(ji  ,jj+1,jk) )    &
                  &                       + 4.*  zww(ji,jj    ,jk)                       )
            END DO
         END DO
         DO jj = 3, jpj-2                               ! other rows
            DO ji = fs_2, fs_jpim1   ! vector opt.
               uslp(ji,jj,jk) = zcofu * (        zwz(ji-1,jj-1,jk) + zwz(ji+1,jj-1,jk)      &
                  &                       +      zwz(ji-1,jj+1,jk) + zwz(ji+1,jj+1,jk)      &
                  &                       + 2.*( zwz(ji  ,jj-1,jk) + zwz(ji-1,jj  ,jk)      &
                  &                       +      zwz(ji+1,jj  ,jk) + zwz(ji  ,jj+1,jk) )    &
                  &                       + 4.*  zwz(ji  ,jj  ,jk)                       )
               vslp(ji,jj,jk) = zcofv * (        zww(ji-1,jj-1,jk) + zww(ji+1,jj-1,jk)      &
                  &                       +      zww(ji-1,jj+1,jk) + zww(ji+1,jj+1,jk)      &
                  &                       + 2.*( zww(ji  ,jj-1,jk) + zww(ji-1,jj  ,jk)      &
                  &                       +      zww(ji+1,jj  ,jk) + zww(ji  ,jj+1,jk) )    &
                  &                       + 4.*  zww(ji,jj    ,jk)                       )
            END DO
         END DO
         !                                        !* decrease along coastal boundaries
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               z1u  = ( umask(ji,jj+1,jk) + umask(ji,jj-1,jk) )*.5
               z1v  = ( vmask(ji+1,jj,jk) + vmask(ji-1,jj,jk) )*.5
               z1wu = ( umask(ji,jj,jk)   + umask(ji,jj,jk+1) )*.5
               z1wv = ( vmask(ji,jj,jk)   + vmask(ji,jj,jk+1) )*.5
               uslp(ji,jj,jk) = uslp(ji,jj,jk) * z1u * z1wu
               vslp(ji,jj,jk) = vslp(ji,jj,jk) * z1v * z1wv
            END DO
         END DO
      END DO


      ! II.  slopes at w point           | wslpi = mij( d/di( prd ) / d/dz( prd )
      ! ===========================      | wslpj = mij( d/dj( prd ) / d/dz( prd )
      !               
      !                                           !* Local vertical density gradient evaluated from N^2
      DO jk = 2, jpkm1                            !  zwy = d/dz(prd)= - mk ( prd ) / grav * pn2 -- at w point
         DO jj = 1, jpj
            DO ji = 1, jpi
               zwy(ji,jj,jk) = zm05g * pn2 (ji,jj,jk) * ( prd (ji,jj,jk) + prd (ji,jj,jk-1) + 2. )
            END DO
         END DO
      END DO
      DO jk = 2, jpkm1                            !* Slopes at w point
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               !                                        ! horizontal density i-gradient at w-points
               zcoef1 = MAX( zeps, umask(ji-1,jj,jk  )+umask(ji,jj,jk  )    &
                  &               +umask(ji-1,jj,jk-1)+umask(ji,jj,jk-1) )
               zcoef1 = 1. / ( zcoef1 * e1t (ji,jj) )
               zai = zcoef1 * (  zgru(ji  ,jj,jk  ) + zgru(ji  ,jj,jk-1)   &
                  &            + zgru(ji-1,jj,jk-1) + zgru(ji-1,jj,jk  ) ) * tmask (ji,jj,jk)
               !                                        ! horizontal density j-gradient at w-points
               zcoef2 = MAX( zeps, vmask(ji,jj-1,jk  )+vmask(ji,jj,jk-1)   &
                  &               +vmask(ji,jj-1,jk-1)+vmask(ji,jj,jk  ) )
               zcoef2 = 1.0 / ( zcoef2 *  e2t (ji,jj) )
               zaj = zcoef2 * (  zgrv(ji,jj  ,jk  ) + zgrv(ji,jj  ,jk-1)   &
                  &            + zgrv(ji,jj-1,jk-1) + zgrv(ji,jj-1,jk  ) ) * tmask (ji,jj,jk)
               !                                        ! bound the slopes: abs(zw.)<= 1/100 and zb..<0.
               !                                        ! static instability: kxz max= ah slope max =< e1 e3 /(pi**2 2 dt)
               zbi = MIN( zwy (ji,jj,jk),- 100.*ABS(zai), -7.e+3/fse3w(ji,jj,jk)*ABS(zai) )
               zbj = MIN( zwy (ji,jj,jk), -100.*ABS(zaj), -7.e+3/fse3w(ji,jj,jk)*ABS(zaj) )
               !                                        ! wslpi and wslpj output in zwz and zww, resp.
               zalpha = MAX( omlmask(ji,jj,jk), omlmask(ji,jj,jk-1) )
               zcoef3 = fsdepw(ji,jj,jk) / MAX( hmlp(ji,jj), 10. )
               zwz(ji,jj,jk) = (     zai / ( zbi - zeps)  * ( 1. - zalpha )   &
                  &             + zcoef3 * wslpiml(ji,jj) *        zalpha   ) * tmask (ji,jj,jk)
               zww(ji,jj,jk) = (     zaj / ( zbj - zeps)  * ( 1. - zalpha )   &
                  &             + zcoef3 * wslpjml(ji,jj) *        zalpha   ) * tmask (ji,jj,jk)
            END DO
         END DO
      END DO
      CALL lbc_lnk( zwz, 'T', -1. )   ;    CALL lbc_lnk( zww, 'T', -1. )      ! lateral boundary conditions on zwz and zww
      !
      !                                           !* horizontal Shapiro filter
      DO jk = 2, jpkm1
         DO jj = 2, jpjm1, jpj-3                        ! rows jj=2 and =jpjm1
            DO ji = 2, jpim1
               zcofw = tmask(ji,jj,jk) / 16.
               wslpi(ji,jj,jk) = (        zwz(ji-1,jj-1,jk) + zwz(ji+1,jj-1,jk)     &
                  &                +      zwz(ji-1,jj+1,jk) + zwz(ji+1,jj+1,jk)     &
                  &                + 2.*( zwz(ji  ,jj-1,jk) + zwz(ji-1,jj  ,jk)     &
                  &                +      zwz(ji+1,jj  ,jk) + zwz(ji  ,jj+1,jk) )   &
                  &                + 4.*  zwz(ji  ,jj  ,jk)                        ) * zcofw

               wslpj(ji,jj,jk) = (        zww(ji-1,jj-1,jk) + zww(ji+1,jj-1,jk)     &
                  &                +      zww(ji-1,jj+1,jk) + zww(ji+1,jj+1,jk)     &
                  &                + 2.*( zww(ji  ,jj-1,jk) + zww(ji-1,jj  ,jk)     &
                  &                +      zww(ji+1,jj  ,jk) + zww(ji  ,jj+1,jk) )   &
                  &                + 4.*  zww(ji  ,jj  ,jk)                        ) * zcofw
            END DO
         END DO  
         DO jj = 3, jpj-2                               ! other rows
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zcofw = tmask(ji,jj,jk) / 16.
               wslpi(ji,jj,jk) = (        zwz(ji-1,jj-1,jk) + zwz(ji+1,jj-1,jk)     &
                  &                +      zwz(ji-1,jj+1,jk) + zwz(ji+1,jj+1,jk)     &
                  &                + 2.*( zwz(ji  ,jj-1,jk) + zwz(ji-1,jj  ,jk)     &
                  &                +      zwz(ji+1,jj  ,jk) + zwz(ji  ,jj+1,jk) )   &
                  &                + 4.*  zwz(ji  ,jj  ,jk)                        ) * zcofw

               wslpj(ji,jj,jk) = (        zww(ji-1,jj-1,jk) + zww(ji+1,jj-1,jk)     &
                  &                +      zww(ji-1,jj+1,jk) + zww(ji+1,jj+1,jk)     &
                  &                + 2.*( zww(ji  ,jj-1,jk) + zww(ji-1,jj  ,jk)     &
                  &                +      zww(ji+1,jj  ,jk) + zww(ji  ,jj+1,jk) )   &
                  &                + 4.*  zww(ji  ,jj  ,jk)                        ) * zcofw
            END DO
         END DO
         !                                        !* decrease along coastal boundaries
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               z1u = ( umask(ji,jj,jk) + umask(ji-1,jj,jk) ) *.5
               z1v = ( vmask(ji,jj,jk) + vmask(ji,jj-1,jk) ) *.5
               wslpi(ji,jj,jk) = wslpi(ji,jj,jk) * z1u * z1v
               wslpj(ji,jj,jk) = wslpj(ji,jj,jk) * z1u * z1v
            END DO
         END DO
      END DO
         
      ! III.  Specific grid points     
      ! =========================== 
      !               
      IF( cp_cfg == "orca" .AND. jp_cfg == 4 ) THEN     !  ORCA_R4 configuration: horizontal diffusion in specific area
         !                                                    ! Gibraltar Strait
         ij0 =  50   ;   ij1 =  53
         ii0 =  69   ;   ii1 =  71   ;   uslp ( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         ij0 =  51   ;   ij1 =  53
         ii0 =  68   ;   ii1 =  71   ;   vslp ( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         ii0 =  69   ;   ii1 =  71   ;   wslpi( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         ii0 =  69   ;   ii1 =  71   ;   wslpj( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         !
         !                                                    ! Mediterrannean Sea
         ij0 =  49   ;   ij1 =  56
         ii0 =  71   ;   ii1 =  90   ;   uslp ( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         ij0 =  50   ;   ij1 =  56
         ii0 =  70   ;   ii1 =  90   ;   vslp ( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         ii0 =  71   ;   ii1 =  90   ;   wslpi( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
         ii0 =  71   ;   ii1 =  90   ;   wslpj( mi0(ii0):mi1(ii1) , mj0(ij0):mj1(ij1) , : ) = 0.e0
      ENDIF

      
      ! IV. Lateral boundary conditions 
      ! ===============================
      CALL lbc_lnk( uslp , 'U', -1. )      ;      CALL lbc_lnk( vslp , 'V', -1. )
      CALL lbc_lnk( wslpi, 'W', -1. )      ;      CALL lbc_lnk( wslpj, 'W', -1. )


      IF(ln_ctl) THEN
         CALL prt_ctl(tab3d_1=uslp , clinfo1=' slp  - u : ', tab3d_2=vslp,  clinfo2=' v : ', kdim=jpk)
         CALL prt_ctl(tab3d_1=wslpi, clinfo1=' slp  - wi: ', tab3d_2=wslpj, clinfo2=' wj: ', kdim=jpk)
      ENDIF
      !
   END SUBROUTINE ldf_slp

   SUBROUTINE ldf_slp_grif ( kt )
     !!----------------------------------------------------------------------
     !!                 ***  ROUTINE ldf_slp_grif  ***
     !!
     !! ** Purpose :   Compute the squared slopes of neutral surfaces (slope
     !!      of iso-pycnal surfaces referenced locally) ('key_traldfiso')
     !!      at W-points using the Griffies quarter-cells.  Also calculates
     !!      alpha and beta at T-points for use in the Griffies isopycnal
     !!      scheme.
     !!
     !! ** Method  :   The slope in the i-direction is computed at U- and
     !!      W-points (uslp, wslpi) and the slope in the j-direction is
     !!      computed at V- and W-points (vslp, wslpj).
     !!
     !! ** Action : - alpha, beta
     !!               wslp2 squared slope of neutral surfaces at w-points.
     !!
     !! History :
     !!   9.0  !  06-10  (C. Harris)  New subroutine
     !!----------------------------------------------------------------------
     !! * Modules used
     USE oce            , zdit  => ua,  &  ! use ua as workspace
          zdis  => va,  &  ! use va as workspace
          zdjt  => ta,  &  ! use ta as workspace
          zdjs  => sa      ! use sa as workspace
     !! * Arguments
     INTEGER, INTENT( in ) ::   kt         ! ocean time-step index

     !! * Local declarations
     INTEGER  ::   ji, jj, jk, ip, jp, kp  ! dummy loop indices
     INTEGER  ::   iku, ikv                ! temporary integer
     REAL(wp) ::   &
          zt, zs, zh, zt2, zsp5, zp1t1,   &  ! temporary scalars
          zdenr, zrhotmp, zdndt, zdddt,   &  !     "        "
          zdnds, zddds, znum, zden,       &  !     "        "
          zslope, za_sxe, zslopec, zdsloper,&  !     "        "
          zfact, zepsln, zatempw,zatempu,zatempv, &   !     "        "
          ze1ur,ze2vr,ze3wr,zdxt,zdxs,zdyt,zdys,zdzt,zdzs,zvolf,&
          zr_slpmax,zdxrho,zdyrho,zabs_dzrho
     REAL(wp), DIMENSION(jpi,jpj,jpk,0:1,0:1) ::   &
          zsx,zsy
     REAL(wp), DIMENSION(jpi,jpj,0:1,0:1) ::   &
          zsx_ml_base,zsy_ml_base
     REAL(wp), DIMENSION(jpi,jpj,jpk) ::   &
          zdkt,zdks
     REAL(wp), DIMENSION(jpi,jpj) ::   &
          zr_ml_basew
     !!----------------------------------------------------------------------

     ! 0. Local constant initialization
     ! --------------------------------
     zr_slpmax = 1.0_wp/slpmax

     ! zslopec=0.004
     ! zdsloper=1000.0
     zepsln=1e-25

     SELECT CASE ( nn_eos )

     CASE ( 0 )               ! Jackett and McDougall (1994) formulation

        !                                                ! ===============
        DO jk = 1, jpk                                   ! Horizontal slab
           !                                             ! ===============
           DO jj = 1, jpjm1
              DO ji = 1, fs_jpim1
                 zt = tb(ji,jj,jk)          ! potential temperature
                 zs = sb(ji,jj,jk) - 35.0   ! salinity anomaly (s-35)
                 zh = fsdept(ji,jj,jk)      ! depth in meters

                 beta(ji,jj,jk) = ( ( -0.415613e-09 * zt + 0.555579e-07 ) * zt      &
                      &                            - 0.301985e-05 ) * zt      &
                      &   + 0.785567e-03                                      &
                      &   + (     0.515032e-08 * zs                           &
                      &         + 0.788212e-08 * zt - 0.356603e-06 ) * zs     &
                      &   +(  (   0.121551e-17 * zh                           &
                      &         - 0.602281e-15 * zs                           &
                      &         - 0.175379e-14 * zt + 0.176621e-12 ) * zh     &
                      &                             + 0.408195e-10   * zs     &
                      &     + ( - 0.213127e-11 * zt + 0.192867e-09 ) * zt     &
                      &                             - 0.121555e-07 ) * zh

                 alpha(ji,jj,jk) = - beta(ji,jj,jk) *                       &
                      &     (((( - 0.255019e-07 * zt + 0.298357e-05 ) * zt   &
                      &                               - 0.203814e-03 ) * zt   &
                      &                               + 0.170907e-01 ) * zt   &
                      &   + 0.665157e-01                                      &
                      &   +     ( - 0.678662e-05 * zs                         &
                      &           - 0.846960e-04 * zt + 0.378110e-02 ) * zs   &
                      &   +   ( ( - 0.302285e-13 * zh                         &
                      &           - 0.251520e-11 * zs                         &
                      &           + 0.512857e-12 * zt * zt           ) * zh   &
                      &           - 0.164759e-06 * zs                         &
                      &        +(   0.791325e-08 * zt - 0.933746e-06 ) * zt   &
                      &                               + 0.380374e-04 ) * zh)

              ENDDO
           ENDDO
        ENDDO

     CASE ( 1 )

        alpha(:,:,:)=-rn_alpha
        beta(:,:,:)=0.0

     CASE ( 2 )

        alpha(:,:,:)=-rn_alpha
        beta (:,:,:)=rn_beta

     CASE ( 3 )

        DO jk =1, jpk
           DO jj = 1, jpjm1
              DO ji = 1, fs_jpim1
                 zt = tb(ji,jj,jk)
                 zs = sb(ji,jj,jk)
                 zh = fsdept(ji,jj,jk)
                 zt2 = zt**2
                 zsp5 = sqrt(ABS(zs))
                 zp1t1=zh*zt
                 znum=9.99843699e+02+zt*(7.35212840e+00+zt*(-5.45928211e-02+3.98476704e-04*zt)) &
                      +zs*(2.96938239e+00-7.23268813e-03*zt+2.12382341e-03*zs)  &
                      +zh*(1.04004591e-02+1.03970529e-07*zt2+5.18761880e-06*zs+ &
                      zh*(-3.24041825e-08-1.23869360e-11*zt2))
                 zden=1.00000000e+00+zt*(7.28606739e-03+zt*(-4.60835542e-05+zt*(3.68390573e-07+zt*1.80809186e-10))) &
                      +zs*(2.14691708e-03+zt*(-9.27062484e-06-1.78343643e-10*zt2)+zsp5*(4.76534122e-06+1.63410736e-09*zt2)) &
                      + zh*(5.30848875e-06+zh*zt*(-3.03175128e-16*zt2-1.27934137e-17*zh))
                 zdenr=1.0/zden
                 zrhotmp=znum*zdenr
                 zdndt=7.35212840e+00+zt*(-1.091856422e-01+1.195430112e-03*zt)-7.23268813e-03*zs &
                      +zp1t1*(2.07941058e-07-2.4773872e-11*zh)
                 zdddt=7.28606739e-03+zt*(-9.21671084e-05+zt*(1.105171719e-06+7.23236744e-10*zt))  &
                      +zs*(-9.27062484e-06+zt*(-5.35030929e-10*zt+3.26821472e-09*zsp5))  &
                      +zh*zh*(-9.09525384e-16*zt2-1.27934137e-17*zh)
                 zdnds=2.96938239e+00-7.23268813e-03*zt+2*2.12382341e-03*zs+5.18761880e-06*zh
                 zddds=2.14691708e-03+zt*(-9.27062484e-06-1.78343643e-10*zt2)+zsp5*(7.14801183e-06+2.45116104e-09*zt2)
                 alpha(ji,jj,jk)=(zdndt-zrhotmp*zdddt)*zdenr
                 beta(ji,jj,jk)=zdenr*(zdnds-zrhotmp*zddds)

              END DO
           END DO
        END DO

     CASE DEFAULT

        IF(lwp) WRITE(numout,cform_err)
        IF(lwp) WRITE(numout,*) '          bad flag value for nn_eos = ', nn_eos
        nstop = nstop + 1

     END SELECT

     CALL lbc_lnk( alpha, 'T', 1. )
     CALL lbc_lnk( beta, 'T', 1. )


     ! ---------------------------------------
     ! 1. Horizontal tracer gradients at T-level jk
     ! ---------------------------------------
     DO jk = 1, jpkm1
        DO jj = 1, jpjm1
           DO ji = 1, fs_jpim1   ! vector opt.
              ! i-gradient of T and S at jj
              zdit (ji,jj,jk) = ( tb(ji+1,jj,jk)-tb(ji,jj,jk) ) * umask(ji,jj,jk)
              zdis (ji,jj,jk) = ( sb(ji+1,jj,jk)-sb(ji,jj,jk) ) * umask(ji,jj,jk)
              ! j-gradient of T and S at jj
              zdjt (ji,jj,jk) = ( tb(ji,jj+1,jk)-tb(ji,jj,jk) ) * vmask(ji,jj,jk)
              zdjs (ji,jj,jk) = ( sb(ji,jj+1,jk)-sb(ji,jj,jk) ) * vmask(ji,jj,jk)
           END DO
        END DO
     END DO

     IF( ln_zps ) THEN      ! partial steps correction at the last level
# if defined key_vectopt_loop   &&   ! defined key_mpp_omp
     jj = 1
        DO ji = 1, jpij-jpi   ! vector opt. (forced unrolling)
# else
           DO jj = 1, jpjm1
              DO ji = 1, jpim1
# endif
                 ! last ocean level
                 iku  = MIN( mbathy(ji,jj), mbathy(ji+1,jj  ) ) - 1
                 ikv  = MIN( mbathy(ji,jj), mbathy(ji  ,jj+1) ) - 1
                 ! i-gradient of T and S
                 zdit (ji,jj,iku) = gtu(ji,jj)
                 zdis (ji,jj,iku) = gsu(ji,jj)
                 ! j-gradient of T and S
                 zdjt (ji,jj,ikv) = gtv(ji,jj)
                 zdjs (ji,jj,ikv) = gsv(ji,jj)
# if ! defined key_vectopt_loop   ||   defined key_mpp_omp
              END DO
# endif
           END DO
        ENDIF

        ! ---------------------------------------
        ! 1. Vertical tracer gradient at w-level jk
        ! ---------------------------------------
        DO jk = 2, jpk
           zdkt(:,:,jk) = ( tb(:,:,jk-1) - tb(:,:,jk) ) * tmask(:,:,jk)
           zdks(:,:,jk) = ( sb(:,:,jk-1) - sb(:,:,jk) ) * tmask(:,:,jk)
        END DO

        zdkt(:,:,1) = 0.0
        zdks(:,:,1) = 0.0

        ! ---------------------------------------
        ! Depth of the w-point below ML base
        ! ---------------------------------------
        DO jj = 1, jpj
           DO ji = 1, jpi
              jk = nmln(ji,jj)
              zr_ml_basew(ji,jj)=1.0/fsdepw(ji,jj,jk+1)
           END DO
        END DO


        wslp2(:,:,:)=0.0
        tfw(:,:,:) = 0.0
        sfw(:,:,:) = 0.0
        ftu(:,:,:) = 0.0
        fsu(:,:,:) = 0.0
        ftv(:,:,:) = 0.0
        fsv(:,:,:) = 0.0
        ftud(:,:,:) = 0.0
        fsud(:,:,:) = 0.0
        ftvd(:,:,:) = 0.0
        fsvd(:,:,:) = 0.0
        psix_eiv(:,:,:) = 0.0
        psiy_eiv(:,:,:) = 0.0

        ! ----------------------------------------------------------------------
        ! x-z plane
        ! ----------------------------------------------------------------------

        ! calculate limited triad x-slopes zsx in interior (1=<jk=<jpk-1)
        DO ip=0,1
           DO kp=0,1

              DO jk = 1, jpkm1
                 DO jj = 1, jpjm1
                    DO ji = 1, fs_jpim1   ! vector opt.

                       ze1ur=1.0/e1u(ji,jj)
                       zdxt=zdit(ji,jj,jk)*ze1ur
                       zdxs=zdis(ji,jj,jk)*ze1ur

                       ze3wr=1.0/fse3w(ji+ip,jj,jk+kp)
                       zdzt=zdkt(ji+ip,jj,jk+kp)*ze3wr
                       zdzs=zdks(ji+ip,jj,jk+kp)*ze3wr
                       ! Calculate the horizontal and vertical density gradient
                       zdxrho = alpha(ji+ip,jj,jk)*zdxt+beta(ji+ip,jj,jk)*zdxs
                       zabs_dzrho = ABS(alpha(ji+ip,jj,jk)*zdzt+beta(ji+ip,jj,jk)*zdzs)+zepsln
                       ! Limit by slpmax, and mask by psi-point
                       zsx(ji+ip,jj,jk,1-ip,kp) = umask(ji,jj,jk+kp) &
                            & *zdxrho/MAX(zabs_dzrho,zr_slpmax*ABS(zdxrho))
                    END DO
                 END DO
              END DO

           END DO
        END DO

        ! calculate slope of triad immediately below mixed-layer base
        DO ip=0,1
           DO kp=0,1
              DO jj = 1, jpjm1
                 DO ji = 1, fs_jpim1
                    jk = nmln(ji+ip,jj)
                    zsx_ml_base(ji+ip,jj,1-ip,kp)=zsx(ji+ip,jj,jk+1-kp,1-ip,kp)
                 END DO
              END DO
           END DO
        END DO

        ! Below ML use limited zsx as is
        ! Inside ML replace by linearly reducing zsx_ml_base towards surface
        DO ip=0,1
           DO kp=0,1

              DO jk = 1, jpkm1

                 DO jj = 1, jpjm1

                    DO ji = 1, fs_jpim1   ! vector opt.
                       ! k index of uppermost point(s) of triad is jk+kp-1
                       ! must be .ge. nmln(ji,jj) for zfact=1.
                       !                   otherwise  zfact=0.
                       zfact = 1 - 1/(1 + (jk+kp-1)/nmln(ji+ip,jj))
                       zsx(ji+ip,jj,jk,1-ip,kp) = zfact*zsx(ji+ip,jj,jk,1-ip,kp) + &
                            & (1.0_wp-zfact)*(fsdepw(ji+ip,jj,jk+kp)*zr_ml_basew(ji+ip,jj))*zsx_ml_base(ji+ip,jj,1-ip,kp) 
                    END DO

                 END DO

              END DO
           END DO
        END DO

        ! Use zsx to calculate fluxes and save averaged slopes psix_eiv at psi-points
        DO ip=0,1
           DO kp=0,1

              DO jk = 1, jpkm1

                 DO jj = 1, jpjm1

                    DO ji = 1, fs_jpim1   ! vector opt.

                       ze1ur=1.0/e1u(ji,jj)
                       zdxt=zdit(ji,jj,jk)*ze1ur
                       zdxs=zdis(ji,jj,jk)*ze1ur

                       ze3wr=1.0/fse3w(ji+ip,jj,jk+kp)
                       zdzt=zdkt(ji+ip,jj,jk+kp)*ze3wr
                       zdzs=zdks(ji+ip,jj,jk+kp)*ze3wr
                       zslope=zsx(ji+ip,jj,jk,1-ip,kp)

                       zvolf = 0.25_wp*e1u(ji,jj)*e2u(ji,jj)*fse3u(ji,jj,jk)

                       ftu(ji,jj,jk)= ftu(ji,jj,jk)+zslope*zdzt*zvolf*ze1ur
                       fsu(ji,jj,jk)= fsu(ji,jj,jk)+zslope*zdzs*zvolf*ze1ur
                       ftud(ji,jj,jk)=ftud(ji,jj,jk)+fsahtu(ji,jj,jk)*zdxt*zvolf*ze1ur
                       fsud(ji,jj,jk)=fsud(ji,jj,jk)+fsahtu(ji,jj,jk)*zdxs*zvolf*ze1ur
                       tfw(ji+ip,jj,jk+kp)=tfw(ji+ip,jj,jk+kp)+(zvolf*ze3wr)*zslope*zdxt
                       sfw(ji+ip,jj,jk+kp)=sfw(ji+ip,jj,jk+kp)+(zvolf*ze3wr)*zslope*zdxs
                       wslp2(ji+ip,jj,jk+kp)=wslp2(ji+ip,jj,jk+kp)+ &
                            & ((zvolf*ze3wr)/(e1t(ji+ip,jj)*e2t(ji+ip,jj)))*(zslope)**2
                       psix_eiv(ji,jj,jk+kp) = psix_eiv(ji,jj,jk+kp) + 0.25_wp*zslope

                    END DO
                 END DO

              END DO
           END DO
        END DO

        ! ----------------------------------------------------------------------
        ! y-z plane
        ! ----------------------------------------------------------------------

        ! calculate limited triad y-slopes zsy in interior (1=<jk=<jpk-1)
        DO jp=0,1
           DO kp=0,1

              DO jk = 1, jpkm1
                 DO jj = 1, jpjm1
                    DO ji = 1, fs_jpim1   ! vector opt.

                       ze2vr=1.0/e2v(ji,jj)
                       zdyt=zdjt(ji,jj,jk)*ze2vr
                       zdys=zdjs(ji,jj,jk)*ze2vr

                       ze3wr=1.0/fse3w(ji,jj+jp,jk+kp)
                       zdzt=zdkt(ji,jj+jp,jk+kp)*ze3wr
                       zdzs=zdks(ji,jj+jp,jk+kp)*ze3wr
                       ! Calculate the horizontal and vertical density gradient
                       zdyrho = alpha(ji,jj+jp,jk)*zdyt+beta(ji,jj+jp,jk)*zdys
                       zabs_dzrho = ABS(alpha(ji,jj+jp,jk)*zdzt+beta(ji,jj+jp,jk)*zdzs)+zepsln
                       ! Limit by slpmax, and mask by psi-point
                       zsy(ji,jj+jp,jk,1-jp,kp) = vmask(ji,jj,jk+kp) &
                            & *zdyrho/MAX(zabs_dzrho,zr_slpmax*ABS(zdyrho))
                    END DO
                 END DO
              END DO

           END DO
        END DO

        ! calculate slope of triad immediately below mixed-layer base
        DO jp=0,1
           DO kp=0,1
              DO jj = 1, jpjm1
                 DO ji = 1, fs_jpim1
                    jk = nmln(ji,jj+jp)
                    zsy_ml_base(ji,jj+jp,1-jp,kp)=zsy(ji,jj+jp,jk+1-kp,1-jp,kp)
                 END DO
              END DO
           END DO
        END DO

        ! Below ML use limited zsy as is
        ! Inside ML replace by linearly reducing zsy_ml_base towards surface
        DO jp=0,1
           DO kp=0,1

              DO jk = 1, jpkm1

                 DO jj = 1, jpjm1

                    DO ji = 1, fs_jpim1   ! vector opt.
                       ! k index of uppermost point(s) of triad is jk+kp-1
                       ! must be .ge. nmln(ji,jj) for zfact=1.
                       !                   otherwise  zfact=0.
                       zfact = 1 - 1/(1 + (jk+kp-1)/nmln(ji,jj+jp))
                       zsy(ji,jj+jp,jk,1-jp,kp) = zfact*zsy(ji,jj+jp,jk,1-jp,kp) + &
                            & (1.0_wp-zfact)*(fsdepw(ji,jj+jp,jk+kp)*zr_ml_basew(ji,jj+jp))*zsy_ml_base(ji,jj+jp,1-jp,kp) 
                    END DO

                 END DO

              END DO
           END DO
        END DO

        ! Use zsy to calculate fluxes and save averaged slopes psiy_eiv at psi-points
        DO jp=0,1
           DO kp=0,1

              DO jk = 1, jpkm1

                 DO jj = 1, jpjm1

                    DO ji = 1, fs_jpim1   ! vector opt.

                       ze2vr=1.0/e2v(ji,jj)
                       zdyt=zdjt(ji,jj,jk)*ze2vr
                       zdys=zdjs(ji,jj,jk)*ze2vr

                       ze3wr=1.0/fse3w(ji,jj+jp,jk+kp)
                       zdzt=zdkt(ji,jj+jp,jk+kp)*ze3wr
                       zdzs=zdks(ji,jj+jp,jk+kp)*ze3wr
                       zslope=zsy(ji,jj+jp,jk,1-jp,kp)

                       zvolf = 0.25_wp*e1v(ji,jj)*e2v(ji,jj)*fse3v(ji,jj,jk)

                       ftv(ji,jj,jk)= ftv(ji,jj,jk)+zslope*zdzt*zvolf*ze2vr
                       fsv(ji,jj,jk)= fsv(ji,jj,jk)+zslope*zdzs*zvolf*ze2vr
                       ftvd(ji,jj,jk)=ftvd(ji,jj,jk)+fsahtv(ji,jj,jk)*zdyt*zvolf*ze2vr
                       fsvd(ji,jj,jk)=fsvd(ji,jj,jk)+fsahtv(ji,jj,jk)*zdys*zvolf*ze2vr
                       tfw(ji,jj+jp,jk+kp)=tfw(ji,jj+jp,jk+kp)+(zvolf*ze3wr)*zslope*zdyt
                       sfw(ji,jj+jp,jk+kp)=sfw(ji,jj+jp,jk+kp)+(zvolf*ze3wr)*zslope*zdys
                       wslp2(ji,jj+jp,jk+kp)=wslp2(ji,jj+jp,jk+kp)+ &
                            & ((zvolf*ze3wr)/(e1t(ji,jj+jp)*e2t(ji,jj+jp)))*(zslope)**2
                       psiy_eiv(ji,jj,jk+kp) = psiy_eiv(ji,jj,jk+kp) + 0.25_wp*zslope

                    END DO
                 END DO

              END DO
           END DO
        END DO

        tfw(:,:,1)=0.0
        sfw(:,:,1)=0.0
        wslp2(:,:,1)=0.0

        CALL lbc_lnk( wslp2, 'W', 1. )
        CALL lbc_lnk( tfw, 'W', 1. )
        CALL lbc_lnk( sfw, 'W', 1. )
        CALL lbc_lnk( ftu, 'U', -1. )
        CALL lbc_lnk( fsu, 'U', -1. )
        CALL lbc_lnk( ftv, 'V', -1. )
        CALL lbc_lnk( fsv, 'V', -1. )
        CALL lbc_lnk( ftud, 'U', -1. )
        CALL lbc_lnk( fsud, 'U', -1. )
        CALL lbc_lnk( ftvd, 'V', -1. )
        CALL lbc_lnk( fsvd, 'V', -1. )
        CALL lbc_lnk( psix_eiv, 'U', -1. )
        CALL lbc_lnk( psiy_eiv, 'V', -1. )


      END SUBROUTINE ldf_slp_grif


   SUBROUTINE ldf_slp_mxl( prd, pn2 )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE ldf_slp_mxl  ***
      !!
      !! ** Purpose :   Compute the slopes of iso-neutral surface just below 
      !!              the mixed layer.
      !!
      !! ** Method :
      !!      The slope in the i-direction is computed at u- and w-points
      !!   (uslp, wslpi) and the slope in the j-direction is computed at
      !!   v- and w-points (vslp, wslpj).
      !!   They are bounded by 1/100 over the whole ocean, and within the
      !!   surface layer they are bounded by the distance to the surface
      !!   ( slope<= depth/l  where l is the length scale of horizontal
      !!   diffusion (here, aht=2000m2/s ==> l=20km with a typical velocity
      !!   of 10cm/s)
      !!
      !! ** Action :   Compute uslp, wslpi, and vslp, wslpj, the i- and  j-slopes
      !!   of now neutral surfaces at u-, w- and v- w-points, resp.
      !!----------------------------------------------------------------------
      USE oce , zgru  => ua   ! ua, va used as workspace and set to hor. 
      USE oce , zgrv  => va   ! density gradient in ldf_slp
      !! 
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in) ::   prd   ! in situ density
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in) ::   pn2   ! Brunt-Vaisala frequency (locally ref.)
      !!
      INTEGER  ::   ji, jj, jk           ! dummy loop indices
      INTEGER  ::   ik, ikm1             ! temporary integers
      REAL(wp) ::   zeps, zmg, zm05g     ! temporary scalars
      REAL(wp) ::   zcoef1, zcoef2       !    -         -
      REAL(wp) ::   zau, zbu, zai, zbi   !    -         -
      REAL(wp) ::   zav, zbv, zaj, zbj   !    -         -
      REAL(wp), DIMENSION(jpi,jpj) ::   zwy   ! 2D workspace
      !!----------------------------------------------------------------------

      zeps  =  1.e-20                    ! Local constant initialization
      zmg   = -1.0 / grav
      zm05g = -0.5 / grav
      !
      uslpml (1,:) = 0.e0      ;      uslpml (jpi,:) = 0.e0
      vslpml (1,:) = 0.e0      ;      vslpml (jpi,:) = 0.e0
      wslpiml(1,:) = 0.e0      ;      wslpiml(jpi,:) = 0.e0
      wslpjml(1,:) = 0.e0      ;      wslpjml(jpi,:) = 0.e0

      !                                  ! surface mixed layer mask
      DO jk = 1, jpk                          ! =1 inside the mixed layer, =0 otherwise
# if defined key_vectopt_loop
         DO jj = 1, 1
            DO ji = 1, jpij   ! vector opt. (forced unrolling)
# else
         DO jj = 1, jpj
            DO ji = 1, jpi
# endif
               ik = nmln(ji,jj) - 1
               IF( jk <= ik ) THEN   ;   omlmask(ji,jj,jk) = 1.e0
               ELSE                  ;   omlmask(ji,jj,jk) = 0.e0
               ENDIF
            END DO
         END DO
      END DO


      ! Slopes of isopycnal surfaces just before bottom of mixed layer
      ! --------------------------------------------------------------
      ! The slope are computed as in the 3D case.
      ! A key point here is the definition of the mixed layer at u- and v-points.
      ! It is assumed to be the maximum of the two neighbouring T-point mixed layer depth.
      ! Otherwise, a n2 value inside the mixed layer can be involved in the computation
      ! of the slope, resulting in a too steep diagnosed slope and thus a spurious eddy
      ! induce velocity field near the base of the mixed layer.
      !-----------------------------------------------------------------------
      !
      zwy(:,jpj) = 0.e0            !* vertical density gradient for u-slope (from N^2)
      zwy(jpi,:) = 0.e0
# if defined key_vectopt_loop
      DO jj = 1, 1
         DO ji = 1, jpij-jpi   ! vector opt. (forced unrolling)
# else
      DO jj = 1, jpjm1
         DO ji = 1, jpim1
# endif
            ik = MAX( 1, nmln(ji,jj) , nmln(ji+1,jj) )       ! avoid spurious recirculation 
            ik = MIN( ik, jpkm1 )                            ! if ik = jpk take jpkm1 values
            zwy(ji,jj) = zmg * ( prd(ji,jj,ik) + 1. ) * (    pn2  (ji,jj,ik) + pn2  (ji,jj,ik+1)     )   &
               &                                      / MAX( tmask(ji,jj,ik) + tmask(ji,jj,ik+1), 1. )
         END DO
      END DO
      CALL lbc_lnk( zwy, 'U', 1. )      ! lateral boundary conditions NO sign change

      !                            !* Slope at u points
# if defined key_vectopt_loop
      DO jj = 1, 1
         DO ji = jpi+2, jpij-jpi-1   ! vector opt. (forced unrolling)
# else
      DO jj = 2, jpjm1
         DO ji = 2, jpim1
# endif
            ! horizontal and vertical density gradient at u-points
            ik = MAX( 1, nmln(ji,jj) , nmln(ji+1,jj) )
            ik = MIN( ik, jpkm1 )
            zau = 1./ e1u(ji,jj) * zgru(ji,jj,ik)
            zbu = 0.5*( zwy(ji,jj) + zwy(ji+1,jj) )
            ! bound the slopes: abs(zw.)<= 1/100 and zb..<0
            !                         kxz max= ah slope max =< e1 e3 /(pi**2 2 dt)
            zbu = MIN( zbu, -100.*ABS(zau), -7.e+3/fse3u(ji,jj,ik)*ABS(zau) )
            ! uslpml
            uslpml (ji,jj) = zau / ( zbu - zeps ) * umask (ji,jj,ik)
         END DO
      END DO
      CALL lbc_lnk( uslpml, 'U', -1. )      ! lateral boundary conditions (i-gradient => sign change)

      !                            !* vertical density gradient for v-slope (from N^2)
# if defined key_vectopt_loop
      DO jj = 1, 1
         DO ji = 1, jpij-jpi   ! vector opt. (forced unrolling)
# else
      DO jj = 1, jpjm1
         DO ji = 1, jpim1
# endif
            ik = MAX( 1, nmln(ji,jj) , nmln(ji,jj+1) )
            ik = MIN( ik, jpkm1 )
            zwy(ji,jj) = zmg * ( prd(ji,jj,ik) + 1. ) * (    pn2  (ji,jj,ik) + pn2  (ji,jj,ik+1)     )   &
               &                                      / MAX( tmask(ji,jj,ik) + tmask(ji,jj,ik+1), 1. )
         END DO
      END DO
      CALL lbc_lnk( zwy, 'V', 1. )      ! lateral boundary conditions NO sign change

      !                            !* Slope at v points
# if defined key_vectopt_loop
      DO jj = 1, 1
         DO ji = jpi+2, jpij-jpi-1   ! vector opt. (forced unrolling)
# else
      DO jj = 2, jpjm1
         DO ji = 2, jpim1
# endif
            ! horizontal and vertical density gradient at v-points
            ik = MAX( 1, nmln(ji,jj) , nmln(ji,jj+1) )
            ik = MIN( ik,jpkm1 )
            zav = 1./ e2v(ji,jj) * zgrv(ji,jj,ik)
            zbv = 0.5*( zwy(ji,jj) + zwy(ji,jj+1) )
            ! bound the slopes: abs(zw.)<= 1/100 and zb..<0
            !                         kxz max= ah slope max =< e1 e3 /(pi**2 2 dt)
            zbv = MIN( zbv, -100.*ABS(zav), -7.e+3/fse3v(ji,jj,ik)*ABS( zav ) )
            ! vslpml
            vslpml (ji,jj) = zav / ( zbv - zeps ) * vmask (ji,jj,ik)
         END DO
      END DO
      CALL lbc_lnk( vslpml, 'V', -1. )      ! lateral boundary conditions (j-gradient => sign change)


      !                            !* vertical density gradient for w-slope (from N^2)
# if defined key_vectopt_loop
      DO jj = 1, 1
         DO ji = 1, jpij   ! vector opt. (forced unrolling)
# else
      DO jj = 1, jpj
         DO ji = 1, jpi
# endif
            ik = nmln(ji,jj) + 1
            ik = MIN( ik, jpk )
            ikm1 = MAX ( 1, ik-1)
            zwy (ji,jj) = zm05g * pn2 (ji,jj,ik) *     &
               &             ( prd (ji,jj,ik) + prd (ji,jj,ikm1) + 2. )
         END DO
      END DO

      !                            !* Slopes at w points
# if defined key_vectopt_loop
      DO jj = 1, 1
         DO ji = jpi+2, jpij-jpi-1   ! vector opt. (forced unrolling)
# else
      DO jj = 2, jpjm1
         DO ji = 2, jpim1
# endif
            ik = nmln(ji,jj) + 1
            ik = MIN( ik, jpk )
            ikm1 = MAX ( 1, ik-1 )
            ! horizontal density i-gradient at w-points
            zcoef1 = MAX( zeps, umask(ji-1,jj,ik  )+umask(ji,jj,ik  )   &
               &               +umask(ji-1,jj,ikm1)+umask(ji,jj,ikm1) )
            zcoef1 = 1. / ( zcoef1 * e1t (ji,jj) )
            zai = zcoef1 * (  zgru(ji  ,jj,ik  ) + zgru(ji  ,jj,ikm1)   &
               &            + zgru(ji-1,jj,ikm1) + zgru(ji-1,jj,ik  ) ) * tmask (ji,jj,ik)
            ! horizontal density j-gradient at w-points
            zcoef2 = MAX( zeps, vmask(ji,jj-1,ik  )+vmask(ji,jj,ikm1)    &
               &               +vmask(ji,jj-1,ikm1)+vmask(ji,jj,ik  ) )
            zcoef2 = 1.0 / ( zcoef2 *  e2t (ji,jj) )
            zaj = zcoef2 * (  zgrv(ji,jj  ,ik  ) + zgrv(ji,jj  ,ikm1)   &
               &            + zgrv(ji,jj-1,ikm1) + zgrv(ji,jj-1,ik  ) ) * tmask (ji,jj,ik)
            ! bound the slopes: abs(zw.)<= 1/100 and zb..<0.
            !                   static instability:
            !                   kxz max= ah slope max =< e1 e3 /(pi**2 2 dt)
            zbi = MIN ( zwy (ji,jj),- 100.*ABS(zai), -7.e+3/fse3w(ji,jj,ik)*ABS(zai) )
            zbj = MIN ( zwy (ji,jj), -100.*ABS(zaj), -7.e+3/fse3w(ji,jj,ik)*ABS(zaj) )
            ! wslpiml and wslpjml
            wslpiml (ji,jj) = zai / ( zbi - zeps) * tmask (ji,jj,ik)
            wslpjml (ji,jj) = zaj / ( zbj - zeps) * tmask (ji,jj,ik)
         END DO
      END DO
      CALL lbc_lnk( wslpiml, 'W', -1. )   ;   CALL lbc_lnk( wslpjml, 'W', -1. )      ! lateral boundary conditions
      !
   END SUBROUTINE ldf_slp_mxl


   SUBROUTINE ldf_slp_init
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE ldf_slp_init  ***
      !!
      !! ** Purpose :   Initialization for the isopycnal slopes computation
      !!
      !! ** Method  :   read the nammbf namelist and check the parameter 
      !!      values called by tra_dmp at the first timestep (nit000)
      !!----------------------------------------------------------------------
      INTEGER ::   ji, jj, jk   ! dummy loop indices
      !!----------------------------------------------------------------------
      
      IF(lwp) THEN    
         WRITE(numout,*)
         WRITE(numout,*) 'ldf_slp : direction of lateral mixing'
         WRITE(numout,*) '~~~~~~~'
      ENDIF

      ! Direction of lateral diffusion (tracers and/or momentum)
      ! ------------------------------
      ! set the slope to zero (even in s-coordinates)

      uslp (:,:,:) = 0.e0
      vslp (:,:,:) = 0.e0
      wslpi(:,:,:) = 0.e0
      wslpj(:,:,:) = 0.e0

      uslpml (:,:) = 0.e0
      vslpml (:,:) = 0.e0
      wslpiml(:,:) = 0.e0
      wslpjml(:,:) = 0.e0

      IF( (ln_traldf_hor .OR. ln_dynldf_hor) .AND. .NOT. (lk_vvl .AND. ln_rstart) ) THEN
         IF(lwp) THEN
            WRITE(numout,*) '          Horizontal mixing in s-coordinate: slope = slope of s-surfaces'
         ENDIF

         ! geopotential diffusion in s-coordinates on tracers and/or momentum
         ! The slopes of s-surfaces are computed once (no call to ldfslp in step)
         ! The slopes for momentum diffusion are i- or j- averaged of those on tracers

         ! set the slope of diffusion to the slope of s-surfaces
         !      ( c a u t i o n : minus sign as fsdep has positive value )
         DO jk = 1, jpk
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  uslp (ji,jj,jk) = -1./e1u(ji,jj) * ( fsdept(ji+1,jj,jk) - fsdept(ji ,jj ,jk) ) * umask(ji,jj,jk)
                  vslp (ji,jj,jk) = -1./e2v(ji,jj) * ( fsdept(ji,jj+1,jk) - fsdept(ji ,jj ,jk) ) * vmask(ji,jj,jk)
                  wslpi(ji,jj,jk) = -1./e1t(ji,jj) * ( fsdepw(ji+1,jj,jk) - fsdepw(ji-1,jj,jk) ) * tmask(ji,jj,jk) * 0.5
                  wslpj(ji,jj,jk) = -1./e2t(ji,jj) * ( fsdepw(ji,jj+1,jk) - fsdepw(ji,jj-1,jk) ) * tmask(ji,jj,jk) * 0.5
               END DO
            END DO
         END DO
         ! Lateral boundary conditions on the slopes
         CALL lbc_lnk( uslp , 'U', -1. )      ;      CALL lbc_lnk( vslp , 'V', -1. )
         CALL lbc_lnk( wslpi, 'W', -1. )      ;      CALL lbc_lnk( wslpj, 'W', -1. )
      ENDIF
      !
   END SUBROUTINE ldf_slp_init

#else
   !!------------------------------------------------------------------------
   !!   Dummy module :                 NO Rotation of lateral mixing tensor
   !!------------------------------------------------------------------------
   LOGICAL, PUBLIC, PARAMETER ::   lk_ldfslp = .FALSE.    !: slopes flag
CONTAINS
   SUBROUTINE ldf_slp( kt, prd, pn2 )        ! Dummy routine
      INTEGER, INTENT(in) :: kt 
      REAL, DIMENSION(:,:,:), INTENT(in) :: prd, pn2
      WRITE(*,*) 'ldf_slp: You should not have seen this print! error?', kt, prd(1,1,1), pn2(1,1,1)
   END SUBROUTINE ldf_slp
#endif

   !!======================================================================
END MODULE ldfslp
